import json
from pathlib import Path
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *


RES_PATH = Path(__file__).parent.parent / "res"


class TitleWidget(QWidget):
    close_signal = pyqtSignal()
    minimize_signal = pyqtSignal()
    maximize_signal = pyqtSignal()
    show_normal_signal = pyqtSignal()

    def __init__(self):
        super(TitleWidget, self).__init__()
        self.maximized_or_normal = "normal"

        self.minimize_btn = QPushButton()
        self.maximize_or_normal_btn = QPushButton()
        self.close_window_btn = QPushButton()

        self.set_up()

    def set_up(self):
        self.set_object_name()
        self.set_style_sheet()
        self.set_widget()
        self.set_signal()
        self.set_layout()

    def set_object_name(self):
        self.setObjectName("windowTitle")
        self.minimize_btn.setObjectName("minimizeBtn")
        self.maximize_or_normal_btn.setObjectName("maximizeNormalBtn")
        self.close_window_btn.setObjectName("closeBtn")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{theme}/title.qss", encoding="utf-8") as f:
            self.setStyleSheet(f.read())

    def set_widget(self):
        self.minimize_btn.setFixedSize(30, 30)
        self.maximize_or_normal_btn.setFixedSize(30, 30)
        self.close_window_btn.setFixedSize(30, 30)
        self.minimize_btn.setIcon(QIcon(str(RES_PATH / "images/minimize.png")))
        self.maximize_or_normal_btn.setIcon(QIcon(str(RES_PATH / "images/maximize.png")))
        self.close_window_btn.setIcon(QIcon(str(RES_PATH / "images/close.png")))

    def set_signal(self):
        self.minimize_btn.clicked.connect(self.minimize)
        self.maximize_or_normal_btn.clicked.connect(self.maximize_or_show_normal)
        self.close_window_btn.clicked.connect(self.close_window)

    def set_layout(self):
        all_h_layout = QHBoxLayout(self)
        right_h_layout = QHBoxLayout()

        right_h_layout.setSpacing(0)
        right_h_layout.addWidget(self.minimize_btn)
        right_h_layout.addWidget(self.maximize_or_normal_btn)
        right_h_layout.addWidget(self.close_window_btn)

        all_h_layout.addStretch()
        all_h_layout.addLayout(right_h_layout)
        all_h_layout.setContentsMargins(5, 8, 5, 2)

    def minimize(self):
        self.minimize_signal.emit()

    def maximize_or_show_normal(self):
        if self.maximized_or_normal != "maximized":
            self.maximize_or_normal_btn.setIcon(QIcon(str(RES_PATH/"images/orisize.png")))
            self.maximized_or_normal = "maximized"
            self.maximize_signal.emit()
        else:
            self.maximize_or_normal_btn.setIcon(QIcon(str(RES_PATH/"images/maximize.png")))
            self.maximized_or_normal = "normal"
            self.show_normal_signal.emit()

    def close_window(self):
        self.close_signal.emit()

    def reload_style_sheet(self):
        self.set_style_sheet()

    def mouseDoubleClickEvent(self, event):
        super(TitleWidget, self).mouseDoubleClickEvent(event)
        self.maximize_or_show_normal()


# PyQt
from PyQt5.QtWidgets import QCheckBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QCheckBox = createWidgetMenuBase(QCheckBox)
PATH_TO_IMG1 = str(Path(__file__).parent.parent.parent / "res/images/circle_black.png").replace("\\", "/")
PATH_TO_IMG2 = str(Path(__file__).parent.parent.parent / "res/images/circle_blue.png").replace("\\", "/")

class QtBoxStyleCheckBox4(QCheckBox):
    def __init__(self):
        super(QtBoxStyleCheckBox4, self).__init__(str(Path(__file__)))
        self.setText("Qt Box")
        self.setStyleSheet("""
        QCheckBox {
            font-size: 15px;
        }
        
        QCheckBox:hover {
            color: rgba(0, 0, 0, 180);
        }

        QCheckBox::indicator {
            width: 20px;
            height: 20px;
            border: none;
        }

        QCheckBox::indicator:unchecked {
            image: url(%s);
        }

        QCheckBox::indicator:checked {
            image: url(%s);
        }
        """ % (PATH_TO_IMG1, PATH_TO_IMG2))
# PyQt

# PySide
# from PySide2.QtWidgets import QCheckBox


# class QtBoxStyleCheckBox4(QCheckBox):
#     def __init__(self):
#         super(QtBoxStyleCheckBox4, self).__init__()
#         self.setText("Qt Box")
#         self.setStyleSheet("""
#         QCheckBox {
#             font-size: 15px;
#         }

#         QCheckBox:hover {
#             color: rgba(0, 0, 0, 180);
#         }

#         QCheckBox::indicator {
#             width: 20px;
#             height: 20px;
#             border: none;
#         }

#         QCheckBox::indicator:unchecked {
#             image: url(%s);
#         }

#         QCheckBox::indicator:checked {
#             image: url(%s);
#         }
#         """ % (PATH_TO_IMG1, PATH_TO_IMG2))
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QSpinBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QSpinBox = createWidgetMenuBase(QSpinBox)
PATH_TO_IMG1 = str(Path(__file__).parent.parent.parent / "res/images/up.png").replace("\\", "/")
PATH_TO_IMG2 = str(Path(__file__).parent.parent.parent / "res/images/down.png").replace("\\", "/")

class QtBoxStyleSpinBox1(QSpinBox):
    def __init__(self):
        super(QtBoxStyleSpinBox1, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setCursor(Qt.PointingHandCursor)
        self.setAttribute(Qt.WA_MacShowFocusRect, False)
        self.setStyleSheet("""
        QSpinBox {
            border: 1px solid lightgray;
            border-radius: 3px;
        }
        
        QSpinBox::up-button {
            width: 14px;
            height: 14px;
            margin: 0px 3px 0px -3px;
            border-image: url(%s);
        }
        
        QSpinBox::up-button:pressed {
            margin-top: 1px;
        }
            
        QSpinBox::down-button {
            width: 14px;
            height: 14px;
            margin: 0px 3px 0px -3px;
            border-image: url(%s);
        }
        
        QSpinBox::down-button:pressed {
            margin-bottom: 1px;
        }
        """ % (PATH_TO_IMG1, PATH_TO_IMG2))
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QSpinBox


# class QtBoxStyleSpinBox1(QSpinBox):
#     def __init__(self):
#         super(QtBoxStyleSpinBox1, self).__init__()
#         self.setFixedSize(150, 30)
#         self.setCursor(Qt.PointingHandCursor)
#         self.setAttribute(Qt.WA_MacShowFocusRect, False)
#         self.setStyleSheet("""
#         QSpinBox {
#             border: 1px solid lightgray;
#             border-radius: 3px;
#         }

#         QSpinBox::up-button {
#             margin: 0px 3px 0px -3px;
#             border-image: url(%s);
#         }

#         QSpinBox::up-button:pressed {
#             margin-top: 1px;
#         }

#         QSpinBox::down-button {
#             margin: 0px 3px 0px -3px;
#             border-image: url(%s);
#         }

#         QSpinBox::down-button:pressed {
#             margin-bottom: 1px;
#         }
#         """ % (PATH_TO_IMG1, PATH_TO_IMG2))
# PySide


import json
from pathlib import Path
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *

RES_PATH = Path(__file__).parent.parent / "res"


class MessageBox(QWidget):
    def __init__(self):
        super(MessageBox, self).__init__()
        self.title_label = QLabel()
        self.content_browser = QTextBrowser()
        self.confirm_btn = QPushButton()
        self.close_btn = QPushButton(self)

        self.start_x = None
        self.start_y = None

        self.bg_color = None

        self.set_up()

    def set_up(self):
        self.set_widget()
        self.set_signal()
        self.set_layout()
        self.set_window_attr()
        self.set_object_name()
        self.set_style_sheet()

    def set_widget(self):
        self.title_label.setAlignment(Qt.AlignCenter)
        self.title_label.setFixedWidth(220)
        self.content_browser.setMinimumHeight(300)
        self.confirm_btn.setText("OK")
        self.confirm_btn.setFixedSize(100, 30)
        self.close_btn.move(288, 2)
        self.close_btn.setFixedSize(30, 30)
        self.close_btn.setIcon(QIcon(str(RES_PATH / "images/close.png")))

    def set_signal(self):
        self.close_btn.clicked.connect(self.close)
        self.confirm_btn.clicked.connect(self.close)

    def set_layout(self):
        all_v_layout = QVBoxLayout(self)
        title_h_layout = QHBoxLayout()
        btn_h_layout = QHBoxLayout()
        title_h_layout.addWidget(self.title_label)
        title_h_layout.setAlignment(Qt.AlignCenter)
        all_v_layout.addLayout(title_h_layout)
        all_v_layout.addSpacing(5)
        all_v_layout.addWidget(self.content_browser)
        all_v_layout.addSpacing(5)
        btn_h_layout.addWidget(self.confirm_btn)
        btn_h_layout.setAlignment(Qt.AlignCenter)
        all_v_layout.addLayout(btn_h_layout)

    def set_window_attr(self):
        self.setFixedSize(320, 450)
        self.setWindowFlag(Qt.FramelessWindowHint)
        self.setAttribute(Qt.WA_TranslucentBackground)

    def set_object_name(self):
        self.setObjectName("messageBox")
        self.title_label.setObjectName("titleLabel")
        self.content_browser.setObjectName("contentBrowser")
        self.confirm_btn.setObjectName("confirmBtn")
        self.close_btn.setObjectName("closeBtn")

    def set_style_sheet(self):
        with open(RES_PATH / "config.json", "r", encoding="utf-8") as f:
            theme = json.loads(f.read())["theme"]

        with open(RES_PATH / f"qss/{theme}/message_box.qss", encoding="utf-8") as f:
            self.setStyleSheet(f.read())

        if theme == "black":
            self.bg_color = "#3b3f42"
        else:
            self.bg_color = "#f2f2f2"

    def mousePressEvent(self, event):
        pass

    def paintEvent(self, event):
        super(MessageBox, self).paintEvent(event)
        painter = QPainter(self)
        painter.setPen(Qt.NoPen)
        painter.setBrush(QColor(self.bg_color))
        painter.setRenderHint(QPainter.Antialiasing)
        painter.drawRoundedRect(self.rect(), 5, 5)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.start_x = event.x()
            self.start_y = event.y()

    def mouseMoveEvent(self, event):
        if self.start_x is None or self.start_y is None:
            return

        dis_x = event.x() - self.start_x
        dis_y = event.y() - self.start_y
        self.move(self.x() + dis_x, self.y() + dis_y)

    def mouseReleaseEvent(self, event):
        self.start_x = None
        self.start_y = None

    def set_title(self, s):
        self.title_label.setText(s)

    def set_plain_text(self, s):
        self.content_browser.setPlainText(s)

    def set_html(self, s):
        self.content_browser.setHtml(s)

    def reload_style_sheet(self):
        self.set_style_sheet()


# PyQt
from PyQt5.QtWidgets import QComboBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QComboBox = createWidgetMenuBase(QComboBox)

class QtBoxFuncComboBox1(QComboBox):
    def __init__(self):
        super(QtBoxFuncComboBox1, self).__init__(str(Path(__file__)))
        self.setEditable(True)
        self.addItems(['1', '2', '3', '4', '5', '6'])
        self.currentTextChanged.connect(self.update_item)

    def update_item(self, txt):
        self.setItemText(self.currentIndex(), txt)
# PyQt

# PySide
# from PySide2.QtWidgets import QComboBox


# class QtBoxFuncComboBox1(QComboBox):
#     def __init__(self):
#         super(QtBoxFuncComboBox1, self).__init__()
#         self.setEditable(True)
#         self.addItems(['1', '2', '3', '4', '5', '6'])
#         self.currentTextChanged.connect(self.update_item)

#     def update_item(self, txt):
#         self.setItemText(self.currentIndex(), txt)
# PySide


# PyQt
from PyQt5.QtWidgets import QLabel
from PyQt5.QtCore import Qt, QPointF
from PyQt5.QtGui import QPixmap, QPainter, QBrush
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLabel = createWidgetMenuBase(QLabel)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/qt.png").replace("\\", "/")

class QtBoxStyleLabel2(QLabel):
    def __init__(self):
        super(QtBoxStyleLabel2, self).__init__(str(Path(__file__)))
        self.setFixedSize(90, 90)
        self.pixmap = QPixmap(PATH_TO_IMG).scaled(self.width(), self.height())

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setPen(Qt.NoPen)
        brush = QBrush(self.pixmap)
        painter.setBrush(brush)

        points = [
            QPointF(10.0, 80.0),
            QPointF(20.0, 10.0),
            QPointF(80.0, 30.0),
            QPointF(90.0, 70.0)
        ]
        painter.drawConvexPolygon(*points)
# PyQt

# PySide
# from PySide2.QtWidgets import QLabel
# from PySide2.QtCore import Qt, QPointF
# from PySide2.QtGui import QPixmap, QPainter, QBrush


# class QtBoxStyleLabel2(QLabel):
#     def __init__(self):
#         super(QtBoxStyleLabel2, self).__init__()
#         self.setFixedSize(90, 90)
#         self.pixmap = QPixmap(PATH_TO_IMG).scaled(self.width(), self.height())

#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.setPen(Qt.NoPen)
#         brush = QBrush(self.pixmap)
#         painter.setBrush(brush)

#         points = [
#             QPointF(10.0, 80.0),
#             QPointF(20.0, 10.0),
#             QPointF(80.0, 30.0),
#             QPointF(90.0, 70.0)
#         ]
#         painter.drawConvexPolygon(points)
# PySide


# PyQt
from PyQt5.QtWidgets import QProgressBar
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QProgressBar = createWidgetMenuBase(QProgressBar)

class QtBoxStyleProgressBar3(QProgressBar):
    def __init__(self):
        super(QtBoxStyleProgressBar3, self).__init__(str(Path(__file__)))
        self.setRange(0, 100)
        self.setValue(80)
        self.setStyleSheet("""
        QProgressBar {
            border: 2px solid white;
            background-color: black;
            padding-left: 2px;
            padding-right: 2px;
            text-align: center;
        }

        QProgressBar::chunk {
            background-color: white;
            margin-top: 2px;
            margin-bottom: 2px;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QProgressBar


# class QtBoxStyleProgressBar3(QProgressBar):
#     def __init__(self):
#         super(QtBoxStyleProgressBar3, self).__init__()
#         self.setRange(0, 100)
#         self.setValue(80)
#         self.setStyleSheet("""
#         QProgressBar {
#             border: 2px solid white;
#             background-color: black;
#             padding-left: 2px;
#             padding-right: 2px;
#             text-align: center;
#         }

#         QProgressBar::chunk {
#             background-color: white;
#             margin-top: 2px;
#             margin-bottom: 2px;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QSlider
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QSlider = createWidgetMenuBase(QSlider)

class QtBoxStyleSlider5(QSlider):
    def __init__(self):
        super(QtBoxStyleSlider5, self).__init__(str(Path(__file__)))
        self.setOrientation(Qt.Horizontal)
        self.setStyleSheet("""
        QSlider::groove:horizontal {
            border: none;
            height: 10px;
            background-color: lightgray;
            border-radius: 5px;
        }

        QSlider::handle:horizontal {
            width: 4px;
            margin: -1px 0px -1px -5px;
            border-radius: 1px;
            background-color: white;
        }

        QSlider::sub-page:horizontal {
            background-color: qlineargradient(x1:0, y1:0, x2:1, y2:1, stop:0 #59969b, stop:1 #04e7fa);
            border-radius: 5px;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QSlider


# class QtBoxStyleSlider5(QSlider):
#     def __init__(self):
#         super(QtBoxStyleSlider5, self).__init__()
#         self.setOrientation(Qt.Horizontal)
#         self.setStyleSheet("""
#         QSlider::groove:horizontal {
#             border: none;
#             height: 10px;
#             background-color: lightgray;
#             border-radius: 5px;
#         }

#         QSlider::handle:horizontal {
#             width: 4px;
#             margin: -1px 0px -1px -5px;
#             border-radius: 1px;
#             background-color: white;
#         }

#         QSlider::sub-page:horizontal {
#             background-color: qlineargradient(x1:0, y1:0, x2:1, y2:1, stop:0 #59969b, stop:1 #04e7fa);
#             border-radius: 5px;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QPushButton, QGraphicsDropShadowEffect
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton9(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton9, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 40)
        self.setText("BUTTON")
        self.add_shadow()

    def add_shadow(self):
        shadow = QGraphicsDropShadowEffect(self)
        shadow.setBlurRadius(15)
        shadow.setOffset(3, 3)
        shadow.setColor(Qt.gray)
        self.setGraphicsEffect(shadow)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QPushButton, QGraphicsDropShadowEffect


# class QtBoxStyleButton9(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton9, self).__init__()
#         self.setFixedSize(150, 40)
#         self.setText("BUTTON")
#         self.add_shadow()

#     def add_shadow(self):
#         shadow = QGraphicsDropShadowEffect(self)
#         shadow.setBlurRadius(15)
#         shadow.setOffset(3, 3)
#         shadow.setColor(Qt.gray)
#         self.setGraphicsEffect(shadow)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLineEdit, QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxStyleLineEdit3(QLineEdit):
    def __init__(self):
        super(QtBoxStyleLineEdit3, self).__init__(str(Path(__file__)))
        self.setAttribute(Qt.WA_MacShowFocusRect, False)
        self.setPlaceholderText("Enter username")
        self.setFixedSize(180, 30)
        self.setStyleSheet("""
        QLineEdit {
            border: 1px solid lightgray;
            border-radius: 5px;
            border-top-left-radius: 10px;
            border-bottom-left-radius: 10px;
            padding-left: 62px;
            padding-right: 5px;
        }
        """)

        self.label = QLabel(self)
        self.label.setText("Qt Box")
        self.label.setFixedSize(60, 30)
        self.label.move(0, 0)
        self.label.setStyleSheet("""
        QLabel {
            background-color: #f8f9fb;
            color: black; 
            font-size: 15px;
            padding-left: 3px;
            border: 1px solid lightgray;
            border-top-left-radius: 10px;
            border-bottom-left-radius: 10px;
        }
        
        QLabel:hover {
            background-color: #ebecf0;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLineEdit, QLabel


# class QtBoxStyleLineEdit3(QLineEdit):
#     def __init__(self):
#         super(QtBoxStyleLineEdit3, self).__init__()
#         self.setAttribute(Qt.WA_MacShowFocusRect, False)
#         self.setPlaceholderText("Enter username")
#         self.setFixedSize(180, 30)
#         self.setStyleSheet("""
#         QLineEdit {
#             border: 1px solid lightgray;
#             border-radius: 5px;
#             border-top-left-radius: 10px;
#             border-bottom-left-radius: 10px;
#             padding-left: 62px;
#             padding-right: 5px;
#         }
#         """)

#         self.label = QLabel(self)
#         self.label.setText("Qt Box")
#         self.label.setFixedSize(60, 30)
#         self.label.move(0, 0)
#         self.label.setStyleSheet("""
#         QLabel {
#             background-color: #f8f9fb;
#             color: black;
#             font-size: 15px;
#             padding-left: 3px;
#             border: 1px solid lightgray;
#             border-top-left-radius: 10px;
#             border-bottom-left-radius: 10px;
#         }

#         QLabel:hover {
#             background-color: #ebecf0;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import QSize
from PyQt5.QtWidgets import QListWidget, QListWidgetItem, QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QListWidget = createWidgetMenuBase(QListWidget)

class QtBoxFuncListWidget1(QListWidget):
    def __init__(self):
        super(QtBoxFuncListWidget1, self).__init__(str(Path(__file__)))
        self.setFixedSize(100, 200)
        self.add_btns()

    def add_btns(self):
        for _ in range(5):
            item = QListWidgetItem()
            item.setSizeHint(QSize(80, 30))
            self.addItem(item)

            btn = QPushButton()
            btn.setText("button")
            self.setItemWidget(item, btn)
# PyQt

# PySide
# from PySide2.QtCore import QSize
# from PySide2.QtWidgets import QListWidget, QListWidgetItem, QPushButton


# class QtBoxFuncListWidget1(QListWidget):
#     def __init__(self):
#         super(QtBoxFuncListWidget1, self).__init__()
#         self.setFixedSize(80, 200)
#         self.add_btns()

#     def add_btns(self):
#         for _ in range(5):
#             item = QListWidgetItem()
#             item.setSizeHint(QSize(100, 30))
#             self.addItem(item)

#             btn = QPushButton()
#             btn.setText("button")
#             self.setItemWidget(item, btn)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QLineEdit, QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/search.png").replace("\\", "/")

class QtBoxStyleLineEdit1(QLineEdit):
    def __init__(self):
        super(QtBoxStyleLineEdit1, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setPlaceholderText("search")

        self.search_btn = QPushButton(self)
        self.search_btn.setIcon(QIcon(PATH_TO_IMG))
        self.search_btn.setCursor(Qt.PointingHandCursor)
        self.search_btn.setFixedSize(self.height(), self.height())
        self.search_btn.move(self.width()-self.search_btn.width(), 0)

        self.setStyleSheet("""
        QPushButton {
            border: none;
        }

        QLineEdit {
            background-color: #232324;
            border: 1px solid gray;
            border-radius: 3px;
            padding-left: 2px;
            padding-right: 25px;
            color: #f0f0f0;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtGui import QIcon
# from PySide2.QtWidgets import QLineEdit, QPushButton


# class QtBoxStyleLineEdit1(QLineEdit):
#     def __init__(self):
#         super(QtBoxStyleLineEdit1, self).__init__()
#         self.setFixedSize(150, 30)
#         self.setPlaceholderText("search")

#         self.search_btn = QPushButton(self)
#         self.search_btn.setIcon(QIcon(PATH_TO_IMG))
#         self.search_btn.setCursor(Qt.PointingHandCursor)
#         self.search_btn.setFixedSize(self.height(), self.height())
#         self.search_btn.move(self.width()-self.search_btn.width(), 0)

#         self.setStyleSheet("""
#         QPushButton {
#             border: none;
#         }

#         QLineEdit {
#             background-color: #232324;
#             border: 1px solid gray;
#             border-radius: 3px;
#             padding-left: 2px;
#             padding-right: 25px;
#             color: #f0f0f0;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLineEdit, QCompleter
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxFuncLineEdit5(QLineEdit):
    def __init__(self):
        super(QtBoxFuncLineEdit5, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setPlaceholderText("Type P or Q")

        comp = QCompleter(["PyQt", "Qt", "Qt Box", "PySide", "Python"])
        comp.setFilterMode(Qt.MatchContains)
        self.setCompleter(comp)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLineEdit, QCompleter


# class QtBoxFuncLineEdit5(QLineEdit):
#     def __init__(self):
#         super(QtBoxFuncLineEdit5, self).__init__()
#         self.setFixedSize(150, 30)
#         self.setPlaceholderText("Type P or Q")

#         comp = QCompleter(["PyQt", "Qt", "Qt Box", "PySide", "Python"])
#         comp.setFilterMode(Qt.MatchContains)
#         self.setCompleter(comp)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLineEdit, QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxStyleLineEdit5(QLineEdit):
    def __init__(self):
        super(QtBoxStyleLineEdit5, self).__init__(str(Path(__file__)))
        self.setAttribute(Qt.WA_MacShowFocusRect, False)
        self.setPlaceholderText("Enter username")
        self.setFixedSize(150, 30)
        self.setStyleSheet("""
        QLineEdit {
            border-top: none;
            border-left: none;
            border-right: none;
            border-bottom: 1px solid black;
            background-color: transparent;
        }
        
        QLineEdit:focus {
            border-bottom: 1px solid #2a70f4;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLineEdit, QPushButton


# class QtBoxStyleLineEdit5(QLineEdit):
#     def __init__(self):
#         super(QtBoxStyleLineEdit5, self).__init__()
#         self.setAttribute(Qt.WA_MacShowFocusRect, False)
#         self.setPlaceholderText("Enter username")
#         self.setFixedSize(150, 30)
#         self.setStyleSheet("""
#         QLineEdit {
#             border-top: none;
#             border-left: none;
#             border-right: none;
#             border-bottom: 1px solid black;
#             background-color: transparent;
#         }

#         QLineEdit:focus {
#             border-bottom: 1px solid #2a70f4;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QSlider
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QSlider = createWidgetMenuBase(QSlider)
PATH_TO_IMG = str(Path(__file__).parent.parent.parent / "res/images/basketball.png").replace("\\", "/")

class QtBoxStyleSlider4(QSlider):
    def __init__(self):
        super(QtBoxStyleSlider4, self).__init__(str(Path(__file__)))
        self.setOrientation(Qt.Horizontal)
        self.setStyleSheet("""
        QSlider::groove:horizontal {
            border: none;
            height: 2px;
            background-color: lightgray;
        }

        QSlider::handle:horizontal {
            border-image: url(%s);
            width: 20px;
            margin: -10px -2px -10px -2px;
        }

        QSlider::sub-page:horizontal {
            background-color: #ef7067;
        }
        """ % (PATH_TO_IMG))
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QSlider


# class QtBoxStyleSlider4(QSlider):
#     def __init__(self):
#         super(QtBoxStyleSlider4, self).__init__()
#         self.setOrientation(Qt.Horizontal)
#         self.setStyleSheet("""
#         QSlider::groove:horizontal {
#             border: none;
#             height: 2px;
#             background-color: lightgray;
#         }

#         QSlider::handle:horizontal {
#             border-image: url(%s);
#             width: 20px;
#             margin: -10px -2px -10px -2px;
#         }

#         QSlider::sub-page:horizontal {
#             background-color: #ef7067;
#         }
#         """ % (PATH_TO_IMG))
# PySide


# PyQt
from PyQt5.QtCore import QThread, pyqtSignal
from PyQt5.QtWidgets import QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxFuncButton1(QPushButton):
    def __init__(self):
        super(QtBoxFuncButton1, self).__init__(str(Path(__file__)))
        self.setText("Send SMS")
        self.clicked.connect(self.start_SMS_thread)

        self.SMS_thread = SMSThread()
        self.SMS_thread.count_signal.connect(self.count_down)
        self.SMS_thread.finished_signal.connect(self.restore)

    def start_SMS_thread(self):
        self.SMS_thread.start()

    def count_down(self, count):
        self.setEnabled(False)
        self.setText(f"Resend ({count})")

    def restore(self):
        self.setEnabled(True)
        self.setText("Send SMS")


class SMSThread(QThread):
    count_signal = pyqtSignal(int)
    finished_signal = pyqtSignal()

    def __init__(self):
        super(SMSThread, self).__init__()

    def run(self):
        self.send_SMS()

        count = 60
        while count >= 1:
            self.count_signal.emit(count)
            count -= 1
            self.sleep(1)

        self.finished_signal.emit()

    def send_SMS(self):
        """send verification code to the user here"""
        ...
# PyQt

# PySide
# from PySide2.QtCore import QThread, Signal
# from PySide2.QtWidgets import QPushButton


# class QtBoxFuncButton1(QPushButton):
#     def __init__(self):
#         super(QtBoxFuncButton1, self).__init__()
#         self.setText("Send SMS")
#         self.clicked.connect(self.start_SMS_thread)

#         self.SMS_thread = SMSThread()
#         self.SMS_thread.count_signal.connect(self.count_down)
#         self.SMS_thread.finished_signal.connect(self.restore)

#     def start_SMS_thread(self):
#         self.SMS_thread.start()

#     def count_down(self, count):
#         self.setEnabled(False)
#         self.setText(f"Resend ({count})")

#     def restore(self):
#         self.setEnabled(True)
#         self.setText("Send SMS")


# class SMSThread(QThread):
#     count_signal = Signal(int)
#     finished_signal = Signal()

#     def __init__(self):
#         super(SMSThread, self).__init__()

#     def run(self):
#         self.send_SMS()

#         count = 60
#         while count >= 1:
#             self.count_signal.emit(count)
#             count -= 1
#             self.sleep(1)

#         self.finished_signal.emit()

#     def send_SMS(self):
#         """send verification code to the user here"""
#         ...
# PySide




from urllib.request import urlopen
from threading import Thread
import ssl


def compare_version():
    try:
        context = ssl._create_unverified_context()
        r = urlopen("https://la-vie-est-belle.github.io/qtbox/CHANGELOG", context=context, timeout=1)
        remote_version = r.readline().decode().strip()
    except Exception as e:
        return

    current_version = "v1.1.3"
    if remote_version != current_version:
        print(f"Qt Box {remote_version} is available. Upgrade it with command `pip install --upgrade qtbox`")


def check_update():
    Thread(target=compare_version).start()


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QListWidget, QListWidgetItem
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QListWidget = createWidgetMenuBase(QListWidget)

class QtBoxStyleListWidget1(QListWidget):
    def __init__(self):
        super(QtBoxStyleListWidget1, self).__init__(str(Path(__file__)))
        self.setFixedSize(200, 200)
        self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

        for i in range(10):
            item = QListWidgetItem()
            item.setText(str(i))
            self.addItem(item)

        self.setStyleSheet("""
        QListView {
            border: none;
            background-color: #edeef3;
        }
        
        QListView::item {
            height: 40px;
            margin: 5px 5px 5px 5px;
            background-color: white;
            border-radius: 6px;
        }
        
        QListView::item:hover{
            background-color: whitesmoke;
        }
        
        QListView::item:selected{
            color: black;
            border: 1px solid lightgray;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QListWidget, QListWidgetItem


# class QtBoxStyleListWidget1(QListWidget):
#     def __init__(self):
#         super(QtBoxStyleListWidget1, self).__init__(str(Path(__file__)))
#         self.setFixedSize(200, 200)
#         self.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
#         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)

#         for i in range(10):
#             item = QListWidgetItem()
#             item.setText(str(i))
#             self.addItem(item)

#         self.setStyleSheet("""
#         QListView {
#             border: none;
#             background-color: #edeef3;
#         }

#         QListView::item {
#             height: 40px;
#             margin: 5px 5px 5px 5px;
#             background-color: white;
#             border-radius: 6px;
#         }

#         QListView::item:hover{
#             background-color: whitesmoke;
#         }

#         QListView::item:selected{
#             color: black;
#             border: 1px solid lightgray;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLineEdit, QMessageBox
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxFuncLineEdit1(QLineEdit):
    def __init__(self):
        super(QtBoxFuncLineEdit1, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setPlaceholderText("press Enter or Return")

    def keyPressEvent(self, event):
        super(QtBoxFuncLineEdit1, self).keyPressEvent(event)
        if event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return:
            QMessageBox.information(self, "Qt Box", "Key Enter or Return pressed")
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLineEdit, QMessageBox


# class QtBoxFuncLineEdit1(QLineEdit):
#     def __init__(self):
#         super(QtBoxFuncLineEdit1, self).__init__()
#         self.setFixedSize(150, 30)
#         self.setPlaceholderText("press Enter or Return")

#     def keyPressEvent(self, event):
#         super(QtBoxFuncLineEdit1, self).keyPressEvent(event)
#         if event.key() == Qt.Key_Enter or event.key() == Qt.Key_Return:
#             QMessageBox.information(self, "Qt Box", "Key Enter or Return pressed")
# PySide


# PyQt
from PyQt5.QtCore import pyqtProperty, QPropertyAnimation
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtGui import QColor
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton12(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton12, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 40)
        self.setText("BUTTON")
        self._color = QColor()

        self.color_anim = QPropertyAnimation(self, b"color")
        self.color_anim.setDuration(6000)
        self.color_anim.setStartValue(QColor(0, 0, 0))
        self.color_anim.setEndValue(QColor(255, 255, 255))
        self.color_anim.start()

    @pyqtProperty(QColor)
    def color(self):
        return self._color

    @color.setter
    def color(self, value):
        self._color = value
        red = value.red()
        green = value.green()
        blue = value.blue()
        self.setStyleSheet(f"background-color: rgb({red}, {green}, {blue})")
# PyQt

# PySide
# from PySide2.QtCore import Property, QPropertyAnimation
# from PySide2.QtWidgets import QPushButton
# from PySide2.QtGui import QColor


# class QtBoxStyleButton12(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton12, self).__init__()
#         self.setFixedSize(150, 40)
#         self.setText("BUTTON")
#         self._color = QColor()

#         self.color_anim = QPropertyAnimation(self, b"color")
#         self.color_anim.setDuration(6000)
#         self.color_anim.setStartValue(QColor(0, 0, 0))
#         self.color_anim.setEndValue(QColor(255, 255, 255))
#         self.color_anim.start()

#     @Property(QColor)
#     def color(self):
#         return self._color

#     @color.setter
#     def color(self, value):
#         self._color = value
#         red = value.red()
#         green = value.green()
#         blue = value.blue()
#         self.setStyleSheet(f"background-color: rgb({red}, {green}, {blue})")
# PySide



# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QPainter, QFont, QFontMetricsF
from PyQt5.QtWidgets import QDial
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QDial = createWidgetMenuBase(QDial)

class QtBoxFuncDial1(QDial):
    def __init__(self):
        super(QtBoxFuncDial1, self).__init__(str(Path(__file__)))
        self.setRange(0, 100)
        self.setWrapping(True)
        self.setNotchTarget(10)
        self.setNotchesVisible(True)

    def paintEvent(self, event):
        super(QtBoxFuncDial1, self).paintEvent(event)
        painter = QPainter(self)
        painter.setPen(Qt.SolidLine)

        font = QFont()
        font.setPixelSize(18)
        painter.setFont(font)
        font_metrics = QFontMetricsF(font)
        text_width = font_metrics.width(str(self.value()))
        painter.drawText(int(self.width()/2-text_width/2), int(self.height()/2), str(self.value()))
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtGui import QPainter, QFont, QFontMetricsF
# from PySide2.QtWidgets import QDial


# class QtBoxFuncDial1(QDial):
#     def __init__(self):
#         super(QtBoxFuncDial1, self).__init__(str(Path(__file__)))
#         self.setRange(0, 100)
#         self.setWrapping(True)
#         self.setNotchTarget(10)
#         self.setNotchesVisible(True)

#     def paintEvent(self, event):
#         super(QtBoxFuncDial1, self).paintEvent(event)
#         painter = QPainter(self)
#         painter.setPen(Qt.SolidLine)

#         font = QFont()
#         font.setPixelSize(18)
#         painter.setFont(font)
#         font_metrics = QFontMetricsF(font)
#         text_width = font_metrics.width(str(self.value()))
#         painter.drawText(int(self.width()/2-text_width/2), int(self.height()/2), str(self.value()))
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QLineEdit
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxStyleLineEdit6(QLineEdit):
    def __init__(self):
        super(QtBoxStyleLineEdit6, self).__init__(str(Path(__file__)))
        self.setMaxLength(1)
        self.setFixedSize(50, 50)
        self.setAlignment(Qt.AlignCenter)
        self.setStyleSheet("""
        QLineEdit {
            font-size: 25px;
            border: 2px solid black;
        }
        """)
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QLineEdit


# class QtBoxStyleLineEdit6(QLineEdit):
#     def __init__(self):
#         super(QtBoxStyleLineEdit6, self).__init__()
#         self.setMaxLength(1)
#         self.setFixedSize(50, 50)
#         self.setAlignment(Qt.AlignCenter)
#         self.setStyleSheet("""
#         QLineEdit {
#             font-size: 25px;
#             border: 2px solid black;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import QPointF
from PyQt5.QtWidgets import QWidget
from PyQt5.QtGui import QPainter, QPen, QColor
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QWidget = createWidgetMenuBase(QWidget)

class QtBoxFuncWidget2(QWidget):
    def __init__(self):
        super(QtBoxFuncWidget2, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 150)

        self.line_gap = 5
        self.start_x = self.width() / 2
        self.start_y = self.height() / 2

        self.light_pen = QPen(QColor(222, 222, 222))
        self.dark_pen = QPen(QColor(222, 222, 222))
        self.dark_pen.setWidth(2)

    def draw_lines(self, painter):
        painter.setPen(self.dark_pen)

        line_count = 0
        is_row_drawing_stopped = False
        is_col_drawing_stopped = False
        x1, x2 = self.start_x, self.start_x
        y1, y2 = self.start_y, self.start_y
        while True:
            if not is_col_drawing_stopped:
                painter.drawLine(QPointF(x1, 0.0), QPointF(x1, self.height()))
                painter.drawLine(QPointF(x2, 0.0), QPointF(x2, self.height()))

            if not is_row_drawing_stopped:
                painter.drawLine(QPointF(0.0, y1), QPointF(self.width(), y1))
                painter.drawLine(QPointF(0.0, y2), QPointF(self.width(), y2))

            x1 -= self.line_gap
            x2 += self.line_gap
            y1 -= self.line_gap
            y2 += self.line_gap
            if x1 <= 0 or x2 >= self.width():
                is_col_drawing_stopped = True
            if y1 <= 0 or y2 >= self.height():
                is_row_drawing_stopped = True
            if is_row_drawing_stopped and is_col_drawing_stopped:
                break

            line_count += 1
            if line_count % 10 == 0:
                painter.setPen(self.dark_pen)
            else:
                painter.setPen(self.light_pen)

    def paintEvent(self, event):
        super(QtBoxFuncWidget2, self).paintEvent(event)
        painter = QPainter(self)
        self.draw_lines(painter)

    def resizeEvent(self, event):
        super(QtBoxFuncWidget2, self).resizeEvent(event)
        self.start_x = self.width() / 2
        self.start_y = self.height() / 2
        self.update()
# PyQt

# PySide
# from PySide2.QtCore import QPointF
# from PySide2.QtWidgets import QWidget
# from PySide2.QtGui import QPainter, QPen, QColor


# class QtBoxFuncWidget2(QWidget):
#     def __init__(self):
#         super(QtBoxFuncWidget2, self).__init__()
#         self.setFixedSize(150, 150)

#         self.line_gap = 5
#         self.start_x = self.width() / 2
#         self.start_y = self.height() / 2

#         self.light_pen = QPen(QColor(222, 222, 222))
#         self.dark_pen = QPen(QColor(222, 222, 222))
#         self.dark_pen.setWidth(2)

#     def draw_lines(self, painter):
#         painter.setPen(self.dark_pen)

#         line_count = 0
#         is_row_drawing_stopped = False
#         is_col_drawing_stopped = False
#         x1, x2 = self.start_x, self.start_x
#         y1, y2 = self.start_y, self.start_y
#         while True:
#             if not is_col_drawing_stopped:
#                 painter.drawLine(QPointF(x1, 0.0), QPointF(x1, self.height()))
#                 painter.drawLine(QPointF(x2, 0.0), QPointF(x2, self.height()))

#             if not is_row_drawing_stopped:
#                 painter.drawLine(QPointF(0.0, y1), QPointF(self.width(), y1))
#                 painter.drawLine(QPointF(0.0, y2), QPointF(self.width(), y2))

#             x1 -= self.line_gap
#             x2 += self.line_gap
#             y1 -= self.line_gap
#             y2 += self.line_gap
#             if x1 <= 0 or x2 >= self.width():
#                 is_col_drawing_stopped = True
#             if y1 <= 0 or y2 >= self.height():
#                 is_row_drawing_stopped = True
#             if is_row_drawing_stopped and is_col_drawing_stopped:
#                 break

#             line_count += 1
#             if line_count % 10 == 0:
#                 painter.setPen(self.dark_pen)
#             else:
#                 painter.setPen(self.light_pen)

#     def paintEvent(self, event):
#         super(QtBoxFuncWidget2, self).paintEvent(event)
#         painter = QPainter(self)
#         self.draw_lines(painter)

#     def resizeEvent(self, event):
#         super(QtBoxFuncWidget2, self).resizeEvent(event)
#         self.start_x = self.width() / 2
#         self.start_y = self.height() / 2
#         self.update()
# PySide


# PyQt
from PyQt5.QtCore import Qt, QRectF
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtGui import QPainter, QPen, QColor
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton8(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton8, self).__init__(str(Path(__file__)))
        self.btn_state = "on"
        self.setFixedSize(80, 40)

    def paintEvent(self, event):
        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        pen = QPen(Qt.NoPen)
        painter.setPen(pen)

        if self.btn_state == "on":
            painter.setBrush(QColor("#60ca5b"))
            rect = QRectF(0, 0, self.width(), self.height())
            painter.drawRoundedRect(rect, self.height()/2, self.height()/2)

            painter.setBrush(Qt.white)
            rect = QRectF(self.width()/2-5, 5, self.width()/2, self.height()-5*2)
            painter.drawRoundedRect(rect, (self.height()-5*2)/2, (self.height()-5*2)/2)

        else:
            painter.setBrush(QColor("#dcdcdc"))
            rect = QRectF(0, 0, self.width(), self.height())
            painter.drawRoundedRect(rect, self.height()/2, self.height()/2)

            painter.setBrush(Qt.white)
            rect = QRectF(5, 5, self.width()/2, self.height()-5*2)
            painter.drawRoundedRect(rect, (self.height()-5*2)/2, (self.height()-5*2)/2)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            if self.btn_state == "on":
                self.btn_state = "off"
            else:
                self.btn_state = "on"
# PyQt

# PySide
# from PySide2.QtCore import Qt, QRectF
# from PySide2.QtWidgets import QPushButton
# from PySide2.QtGui import QPainter, QPen, QColor


# class QtBoxStyleButton8(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton8, self).__init__()
#         self.btn_state = "on"
#         self.setFixedSize(80, 40)

#     def paintEvent(self, event):
#         painter = QPainter(self)
#         painter.setRenderHint(QPainter.Antialiasing)
#         pen = QPen(Qt.NoPen)
#         painter.setPen(pen)

#         if self.btn_state == "on":
#             painter.setBrush(QColor("#60ca5b"))
#             rect = QRectF(0, 0, self.width(), self.height())
#             painter.drawRoundedRect(rect, self.height()/2, self.height()/2)

#             painter.setBrush(Qt.white)
#             rect = QRectF(self.width()/2-5, 5, self.width()/2, self.height()-5*2)
#             painter.drawRoundedRect(rect, (self.height()-5*2)/2, (self.height()-5*2)/2)

#         else:
#             painter.setBrush(QColor("#dcdcdc"))
#             rect = QRectF(0, 0, self.width(), self.height())
#             painter.drawRoundedRect(rect, self.height()/2, self.height()/2)

#             painter.setBrush(Qt.white)
#             rect = QRectF(5, 5, self.width()/2, self.height()-5*2)
#             painter.drawRoundedRect(rect, (self.height()-5*2)/2, (self.height()-5*2)/2)

#     def mousePressEvent(self, event):
#         if event.button() == Qt.LeftButton:
#             if self.btn_state == "on":
#                 self.btn_state = "off"
#             else:
#                 self.btn_state = "on"
# PySide


# PyQt
from PyQt5.QtWidgets import QProgressBar
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QProgressBar = createWidgetMenuBase(QProgressBar)

class QtBoxStyleProgressBar5(QProgressBar):
    def __init__(self):
        super(QtBoxStyleProgressBar5, self).__init__(str(Path(__file__)))
        self.setTextVisible(False)
        self.setRange(0, 100)
        self.setValue(80)
        self.setStyleSheet("""
        QProgressBar {
            border: 1px solid lightgray;
            border-radius: 3px;
            background-color: qlineargradient(x1:1, y1:0, x2:1, y2:0.4, stop:0 white, stop:1 #dedede);
        }

        QProgressBar::chunk {
            background-color: qlineargradient(x1:1, y1:0, x2:1, y2:0.4, stop:0 white, stop:1 #64d651);
            border-radius： 2px;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QProgressBar


# class QtBoxStyleProgressBar5(QProgressBar):
#     def __init__(self):
#         super(QtBoxStyleProgressBar5, self).__init__()
#         self.setTextVisible(False)
#         self.setRange(0, 100)
#         self.setValue(80)
#         self.setStyleSheet("""
#         QProgressBar {
#             border: 1px solid lightgray;
#             border-radius: 3px;
#             background-color: qlineargradient(x1:1, y1:0, x2:1, y2:0.4, stop:0 white, stop:1 #dedede);
#         }

#         QProgressBar::chunk {
#             background-color: qlineargradient(x1:1, y1:0, x2:1, y2:0.4, stop:0 white, stop:1 #64d651);
#             border-radius： 2px;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QPushButton
from PyQt5.QtGui import QPainter, QPen, QColor, QPainterPath, QFont, QFontMetricsF
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QPushButton = createWidgetMenuBase(QPushButton)

class QtBoxStyleButton4(QPushButton):
    def __init__(self):
        super(QtBoxStyleButton4, self).__init__(str(Path(__file__)))
        self.btn_state = "normal"
        self.setFixedSize(90, 90)
        self.setAttribute(Qt.WA_TranslucentBackground)

        self.setStyleSheet("""
        QPushButton {
            font-size: 20px;;
            font-weight: bold;
        }
        """)

    def paintEvent(self, event):
        painter = QPainter(self)

        if self.btn_state == "normal":
            painter.setBrush(QColor("#3b3f42"))
            pen = QPen(Qt.SolidLine)
            pen.setColor(Qt.white)
            pen .setWidth(3)
            painter.setPen(pen)

        elif self.btn_state == "hover":
            painter.setBrush(Qt.black)
            pen = QPen(Qt.SolidLine)
            pen.setColor(Qt.white)
            pen.setWidth(3)
            painter.setPen(pen)

        elif self.btn_state == "press":
            painter.setBrush(Qt.white)
            pen = QPen(Qt.SolidLine)
            pen.setColor(Qt.black)
            pen.setWidth(3)
            painter.setPen(pen)

        path = QPainterPath()
        path.moveTo(self.rect().left()+self.rect().width()/2, self.rect().top())
        path.lineTo(self.rect().bottomLeft())
        path.lineTo(self.rect().bottomRight())
        path.lineTo(self.rect().left()+self.rect().width()/2, self.rect().top())
        painter.drawPath(path)

        font = QFont()
        font.setBold(True)
        font.setPixelSize(20)
        painter.setFont(font)
        font_metrics = QFontMetricsF(font)
        text_width = font_metrics.width("BTN")
        painter.drawText(int(self.rect().width()/2-text_width/2), int(self.rect().height()/2+20), "BTN")

    def mousePressEvent(self, event):
        self.btn_state = "press"
        self.update()

    def mouseReleaseEvent(self, event):
        self.btn_state = "hover"
        self.update()

    def enterEvent(self, event):
        self.btn_state = "hover"
        self.update()

    def leaveEvent(self, event):
        self.btn_state = "normal"
        self.update()
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtWidgets import QPushButton
# from PySide2.QtGui import QPainter, QPen, QColor, QPainterPath, QFont, QFontMetricsF


# class QtBoxStyleButton4(QPushButton):
#     def __init__(self):
#         super(QtBoxStyleButton4, self).__init__()
#         self.btn_state = "normal"
#         self.setFixedSize(90, 90)
#         self.setAttribute(Qt.WA_TranslucentBackground)

#         self.setStyleSheet("""
#         QPushButton {
#             font-size: 20px;;
#             font-weight: bold;
#         }
#         """)

#     def paintEvent(self, event):
#         painter = QPainter(self)

#         if self.btn_state == "normal":
#             painter.setBrush(QColor("#3b3f42"))
#             pen = QPen(Qt.SolidLine)
#             pen.setColor(Qt.white)
#             pen .setWidth(3)
#             painter.setPen(pen)

#         elif self.btn_state == "hover":
#             painter.setBrush(Qt.black)
#             pen = QPen(Qt.SolidLine)
#             pen.setColor(Qt.white)
#             pen.setWidth(3)
#             painter.setPen(pen)

#         elif self.btn_state == "press":
#             painter.setBrush(Qt.white)
#             pen = QPen(Qt.SolidLine)
#             pen.setColor(Qt.black)
#             pen.setWidth(3)
#             painter.setPen(pen)

#         path = QPainterPath()
#         path.moveTo(self.rect().left()+self.rect().width()/2, self.rect().top())
#         path.lineTo(self.rect().bottomLeft())
#         path.lineTo(self.rect().bottomRight())
#         path.lineTo(self.rect().left()+self.rect().width()/2, self.rect().top())
#         painter.drawPath(path)

#         font = QFont()
#         font.setBold(True)
#         font.setPixelSize(20)
#         painter.setFont(font)
#         font_metrics = QFontMetricsF(font)
#         text_width = font_metrics.width("BTN")
#         painter.drawText(int(self.rect().width()/2-text_width/2), int(self.rect().height()/2+20), "BTN")

#     def mousePressEvent(self, event):
#         self.btn_state = "press"
#         self.update()

#     def mouseReleaseEvent(self, event):
#         self.btn_state = "hover"
#         self.update()

#     def enterEvent(self, event):
#         self.btn_state = "hover"
#         self.update()

#     def leaveEvent(self, event):
#         self.btn_state = "normal"
#         self.update()
# PySide


# PyQt
from PyQt5.QtCore import Qt
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QLineEdit, QPushButton
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)
PATH_TO_OPEN_EYE_IMG = str(Path(__file__).parent.parent.parent / "res/images/open_eye.png").replace("\\", "/")
PATH_TO_CLOSED_EYE_IMG = str(Path(__file__).parent.parent.parent / "res/images/closed_eye.png").replace("\\", "/")

class QtBoxFuncLineEdit4(QLineEdit):
    def __init__(self):
        super(QtBoxFuncLineEdit4, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 30)
        self.setEchoMode(QLineEdit.Password)
        self.setPlaceholderText("Enter password")
        self.setStyleSheet("""
        QLineEdit {
            padding-right: 25px;
        }
        """)

        self.eye_btn = QPushButton(self)
        self.eye_btn.setIcon(QIcon(PATH_TO_CLOSED_EYE_IMG))
        self.eye_btn.setFixedSize(self.height(), self.height())
        self.eye_btn.move(self.width()-self.eye_btn.width(), 0)
        self.eye_btn.clicked.connect(self.change_echo_mode)
        self.eye_btn.setCursor(Qt.PointingHandCursor)
        self.eye_btn.setStyleSheet("""
        QPushButton {
            border: none;
        }
        """)

    def change_echo_mode(self):
        if self.echoMode() == QLineEdit.Normal:
            self.setEchoMode(QLineEdit.Password)
            self.eye_btn.setIcon(QIcon(PATH_TO_CLOSED_EYE_IMG))
        else:
            self.setEchoMode(QLineEdit.Normal)
            self.eye_btn.setIcon(QIcon(PATH_TO_OPEN_EYE_IMG))
# PyQt

# PySide
# from PySide2.QtCore import Qt
# from PySide2.QtGui import QIcon
# from PySide2.QtWidgets import QLineEdit, QPushButton


# class QtBoxFuncLineEdit4(QLineEdit):
#     def __init__(self):
#         super(QtBoxFuncLineEdit4, self).__init__(str(Path(__file__)))
#         self.setFixedSize(150, 30)
#         self.setEchoMode(QLineEdit.Password)
#         self.setPlaceholderText("Enter password")
#         self.setStyleSheet("""
#         QLineEdit {
#             padding-right: 25px;
#         }
#         """)

#         self.eye_btn = QPushButton(self)
#         self.eye_btn.setIcon(QIcon(PATH_TO_CLOSED_EYE_IMG))
#         self.eye_btn.setFixedSize(self.height(), self.height())
#         self.eye_btn.move(self.width()-self.eye_btn.width(), 0)
#         self.eye_btn.clicked.connect(self.change_echo_mode)
#         self.eye_btn.setCursor(Qt.PointingHandCursor)
#         self.eye_btn.setStyleSheet("""
#         QPushButton {
#             border: none;
#         }
#         """)

#     def change_echo_mode(self):
#         if self.echoMode() == QLineEdit.Normal:
#             self.setEchoMode(QLineEdit.Password)
#             self.eye_btn.setIcon(QIcon(PATH_TO_CLOSED_EYE_IMG))
#         else:
#             self.setEchoMode(QLineEdit.Normal)
#             self.eye_btn.setIcon(QIcon(PATH_TO_OPEN_EYE_IMG))
# PySide


# PyQt
from PyQt5.QtWidgets import QDial
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QDial = createWidgetMenuBase(QDial)

class QtBoxStyleDial1(QDial):
    def __init__(self):
        super(QtBoxStyleDial1, self).__init__(str(Path(__file__)))
        self.setStyleSheet("""
        QDial {
            background-color: gray;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QDial


# class QtBoxStyleDial1(QDial):
#     def __init__(self):
#         super(QtBoxStyleDial1, self).__init__(str(Path(__file__)))
#         self.setStyleSheet("""
#         QDial {
#             background-color: gray;
#         }
#         """)
# PySide


# PyQt
from PyQt5.QtCore import QTimer, QTime
from PyQt5.QtWidgets import QLCDNumber
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLCDNumber = createWidgetMenuBase(QLCDNumber)

class QtBoxFuncLCDNumber1(QLCDNumber):
    def __init__(self):
        super(QtBoxFuncLCDNumber1, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 50)
        self.setDigitCount(8)
        self.update_display()

        self.timer = QTimer()
        self.timer.timeout.connect(self.update_display)
        self.timer.start(1000)

    def update_display(self):
        self.display(QTime.currentTime().toString())
# PyQt

# PySide
# from PySide2.QtCore import QTimer, QTime
# from PySide2.QtWidgets import QLCDNumber


# class QtBoxFuncLCDNumber1(QLCDNumber):
#     def __init__(self):
#         super(QtBoxFuncLCDNumber1, self).__init__()
#         self.setFixedSize(150, 50)
#         self.setDigitCount(8)
#         self.update_display()

#         self.timer = QTimer()
#         self.timer.timeout.connect(self.update_display)
#         self.timer.start(1000)

#     def update_display(self):
#         self.display(QTime.currentTime().toString())
# PySide


# PyQt
from PyQt5.QtWidgets import QLineEdit, QLabel
from qtbox.utils.menu import createWidgetMenuBase
from pathlib import Path

QLineEdit = createWidgetMenuBase(QLineEdit)

class QtBoxStyleLineEdit2(QLineEdit):
    def __init__(self):
        super(QtBoxStyleLineEdit2, self).__init__(str(Path(__file__)))
        self.setFixedSize(150, 50)
        self.setStyleSheet("""
        QLineEdit {
            border: 1px solid gray;
            border-radius: 5px;
            padding-top: 14px;
            padding-left: 2px;
            padding-right: 2px;
        }
        """)

        self.label = QLabel(self)
        self.label.setText("Name")
        self.label.move(5, 16)
        self.label.setStyleSheet("""
        QLabel {
            color: gray;
            font-size: 16px;
        }
        """)

    def focusInEvent(self, event):
        super(QtBoxStyleLineEdit2, self).focusInEvent(event)
        self.label.move(5, 5)
        self.label.setStyleSheet("""
        QLabel {
            color: black;
            font-size: 12px;
        }
        """)

    def focusOutEvent(self, event):
        super(QtBoxStyleLineEdit2, self).focusOutEvent(event)
        if self.text():
            return

        self.label.move(5, 16)
        self.label.setStyleSheet("""
        QLabel {
            color: gray;
            font-size: 16px;
        }
        """)
# PyQt

# PySide
# from PySide2.QtWidgets import QLineEdit, QLabel


# class QtBoxStyleLineEdit2(QLineEdit):
#     def __init__(self):
#         super(QtBoxStyleLineEdit2, self).__init__()
#         self.setFixedSize(150, 50)
#         self.setStyleSheet("""
#         QLineEdit {
#             border: 1px solid gray;
#             border-radius: 5px;
#             padding-top: 14px;
#             padding-left: 2px;
#             padding-right: 2px;
#         }
#         """)

#         self.label = QLabel(self)
#         self.label.setText("Name")
#         self.label.move(5, 16)
#         self.label.setStyleSheet("""
#         QLabel {
#             color: gray;
#             font-size: 16px;
#         }
#         """)

#     def focusInEvent(self, event):
#         super(QtBoxStyleLineEdit2, self).focusInEvent(event)
#         self.label.move(5, 5)
#         self.label.setStyleSheet("""
#         QLabel {
#             color: black;
#             font-size: 12px;
#         }
#         """)

#     def focusOutEvent(self, event):
#         super(QtBoxStyleLineEdit2, self).focusOutEvent(event)
#         if self.text():
#             return

#         self.label.move(5, 16)
#         self.label.setStyleSheet("""
#         QLabel {
#             color: gray;
#             font-size: 16px;
#         }
#         """)
# PySide